@@ -34,6 +34,7 @@
 #include "libgcpcanvas/gcp-canvas-group.h"
 #include "libgcpcanvas/gcp-canvas-line.h"
 #include <cmath>
+#include <cstring>
 
 gcpReactionArrow::gcpReactionArrow (gcpReaction* react, unsigned Type): gcpArrow (ReactionArrowType)
 {
@@ -41,6 +42,7 @@ gcpReactionArrow::gcpReactionArrow (gcpReaction* react, unsigned Type): gcpArrow
 	m_Type = Type;
 	m_Start = m_End = NULL;
 	if (react) react->AddChild (this);
+	m_TypeChanged = false;
 }
 
 
@@ -134,6 +136,8 @@ bool gcpReactionArrow::Load (xmlNodePtr node)
 void gcpReactionArrow::Add (GtkWidget* w)
 {
 	gcpWidgetData* pData = (gcpWidgetData*) g_object_get_data (G_OBJECT (w), "data");
+	if (pData->Items[this] != NULL)
+		return;
 	gcpTheme *pTheme = pData->View->GetDoc ()->GetTheme ();
 	GnomeCanvasPoints *points = gnome_canvas_points_new (2);
 	GnomeCanvasGroup* group = GNOME_CANVAS_GROUP(gnome_canvas_item_new (pData->Group, gnome_canvas_group_ext_get_type (), NULL));
@@ -285,6 +289,7 @@ void gcpReactionArrow::Update (GtkWidget* w)
 								NULL);
 			break;
 		case gcpReversibleArrow:
+		case gcpFullReversibleArrow:
 			double dAngle = atan (- m_height / m_width);
 			if (m_width < 0) dAngle += M_PI;
 			points->coords[0] = m_x * pTheme->GetZoomFactor () - pTheme->GetArrowDist () / 2 * sin (dAngle);
